@@ -21,6 +21,13 @@ SANE_Word sanei_epson_usb_product_ids [] =
 	0x11e, 	/* Perfection 1660 */
 	0x801,	/* CX-5200 */
 	0x802,	/* CX-3200 */
-	0	/* last entry */
+	0	/* last entry - this is used for devices that are specified 
+		   in the config file as "usb <vendor> <product>" */
 };
 
+
+
+int sanei_epson_getNumberOfUSBProductIds()
+{
+	return sizeof(sanei_epson_usb_product_ids)/sizeof(SANE_Word);
+}
