@@ -194,6 +194,8 @@ static scsiblk inquiryB = { inquiryC, sizeof (inquiryC) };
 #define get_IN_brightness_steps(in)        getnbyte(in+0x52, 1)
 #define get_IN_threshold_steps(in)         getnbyte(in+0x53, 1)
 #define get_IN_contrast_steps(in)          getnbyte(in+0x54, 1)
+#define get_IN_num_gamma(in)               getbitfield(in+0x57, 15, 4)
+#define get_IN_num_gamma_download(in)      getbitfield(in+0x57, 15, 0)
 #define get_IN_ipc_bw_reverse(in)          getbitfield(in+0x58, 1, 7)
 #define get_IN_ipc_auto1(in)               getbitfield(in+0x58, 1, 6)
 #define get_IN_ipc_auto2(in)               getbitfield(in+0x58, 1, 5)
@@ -228,6 +230,16 @@ static unsigned char set_windowC[] =
   { SET_WINDOW, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, };
 /* opcode,  lun,  _____4 X reserved____,  transfer length, control byte */
 static scsiblk set_windowB = { set_windowC, sizeof (set_windowC) };
+#define set_SW_xferlen(sb, len) putnbyte(sb + 0x06, len, 3)
+
+	/* With the fi-series scanners, we have to use a 12-byte command
+	 * instead of a 10-byte command when communicating via USB.  This
+	 * may be a firmware bug. */
+static unsigned char set_usb_windowC[] =
+  { SET_WINDOW, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+    0x00, 0x00 };
+/* opcode,  lun,  _____4 X reserved____,  transfer length, control byte */
+static scsiblk set_usb_windowB = { set_usb_windowC, sizeof (set_usb_windowC) };
 #define set_SW_xferlen(sb, len) putnbyte(sb + 0x06, len, 3)
 
 /* ==================================================================== */
@@ -378,6 +390,17 @@ static scsiblk mode_select_headerB = {
 };
 
 
+	/* With the fi-series scanners, we have to use a 10-byte header
+	 * instead of a 4-byte header when communicating via USB.  This
+	 * may be a firmware bug. */
+static unsigned char mode_select_usb_headerC[] = {
+  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
+};
+static scsiblk mode_select_usb_headerB = {
+  mode_select_usb_headerC, sizeof (mode_select_usb_headerC)
+};
+
+
 static unsigned char mode_select_parameter_blockC[] = {
   0x00, 0x00, 0x00,
   0x00, 0x00, 0x00,
@@ -893,12 +916,11 @@ static scsiblk window_descriptor_blockB =
 
 /* ==================================================================== */
 
-/*
   static unsigned char request_senseC[] =
-  {REQUEST_SENSE, 0x00, 0x00, 0x00, 0x00, 0x00};
+  {REQUEST_SENSE, 0x00, 0x00, 0x00, 0x12, 0x00,
+   0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
   static scsiblk request_senseB =
   {request_senseC, sizeof (request_senseC)};
-*/
 
 #define set_RS_allocation_length(sb,val) sb[0x04] = (unsigned char)val
 /* defines for request sense return block */
@@ -912,7 +934,8 @@ static scsiblk window_descriptor_blockB =
 #define get_RS_additional_length(b)       b[0x07]	/* always 10 */
 #define get_RS_ASC(b)                     b[0x0c]
 #define get_RS_ASCQ(b)                    b[0x0d]
-#define get_RS_SKSV(b)                    getbitfield(b+0x0f,1,7)	/* valid, always 0 */
+#define get_RS_SKSV(b)                    getbitfield(b+0x0f,1,7)   /* valid, always 0 */
+#define get_RS_offending_byte(b)          getnbyte(b+0x10, 2)
 
 #define rs_return_block_size              18	/* Says Nikon */
 
