@@ -1,7 +1,7 @@
 /* sane - Scanner Access Now Easy.
 
    Copyright (C) 2002 Sergey Vlasov <vsu@altlinux.ru>
-   Copyright (C) 2002 Henning Meier-Geinitz <henning@meier-geinitz.de>
+   Copyright (C) 2002, 2003 Henning Meier-Geinitz <henning@meier-geinitz.de>
    
    This file is part of the SANE package.
    
@@ -52,6 +52,9 @@
 static GT68xx_Command_Set mustek_gt6816_command_set = {
   "mustek-gt6816",		/* Name of this command set */
 
+  0x40,				/* Request type */
+  0x01,				/* Request */
+  
   0x200c,			/* Memory read - wValue */
   0x200b,			/* Memory write - wValue */
 
@@ -87,6 +90,10 @@ static GT68xx_Command_Set mustek_gt6816_command_set = {
 static GT68xx_Command_Set mustek_gt6801_command_set = {
   "mustek-gt6801",
 
+
+  0x40,				/* Request type */
+  0x01,				/* Request */
+
   0x200a,			/* Memory read - wValue */
   0x2009,			/* Memory write - wValue */
 
@@ -122,6 +129,9 @@ static GT68xx_Command_Set mustek_gt6801_command_set = {
 static GT68xx_Command_Set plustek_gt6801_command_set = {
   "plustek-gt6801",
 
+  0x40,				/* Request type */
+  0x04,				/* Request */
+
   0x200a,			/* Memory read - wValue */
   0x2009,			/* Memory write - wValue */
 
@@ -269,7 +279,7 @@ static GT68xx_Model mustek_1200ta_model = {
   SANE_TRUE,			/* Use base_ydpi for all resolutions */
 
   {600, 300, 150, 75, 50, 0},	/* possible x-resolutions */
-  {1200, 600, 300, 150, 100, 50, 0},	/* possible y-resolutions */
+  {1200, 600, 300, 150, 75, 50, 0},	/* possible y-resolutions */
   {16, 12, 8, 0},		/* possible depths in gray mode */
   {16, 12, 8, 0},		/* possible depths in color mode */
 
@@ -327,7 +337,7 @@ static GT68xx_Model mustek_1200cuplus_model = {
   SANE_FIX (217.0),		/* Size of scan area in mm (x) */
   SANE_FIX (299.0),		/* Size of scan area in mm (y) */
 
-  SANE_FIX (5.0),		/* Start of white strip in mm (y) */
+  SANE_FIX (9.0),		/* Start of white strip in mm (y) */
   SANE_FIX (0.0),		/* Start of black mark in mm (x) */
 
   SANE_FIX (0.0),		/* Start of scan area in TA mode in mm (x) */
@@ -346,8 +356,8 @@ static GT68xx_Model mustek_1200cuplus_model = {
   SANE_FIX (2.0),		/* Default gamma value */
 
   SANE_TRUE,			/* Is this a CIS scanner? */
-  0				/* Which flags are needed for this scanner? */
-    /* Everything untested */
+  GT68XX_FLAG_NO_STOP		/* Which flags are needed for this scanner? */
+    /* Tested by Hamersky Robert r.hamersky at utanet.at */
 };
 
 static GT68xx_Model mustek_2400cuplus_model = {
@@ -691,9 +701,10 @@ static GT68xx_Model mustek_a3usb_model = {
   {0x157, 0x157, 0x157},	/* Default exposure parameters */
   SANE_FIX (2.0),		/* Default gamma value */
 
-  SANE_TRUE,			/* Is this a CIS scanner? */
+  SANE_FALSE,			/* Is this a CIS scanner? */
   0				/* Which flags are needed for this scanner? */
-    /* Tested by Pedro Morais <morais@inocam.com> */
+    /* Tested by Pedro Morais <morais@inocam.com>, changes to CCD by hmg, white
+       strip/black mark untested */
 };
 
 static GT68xx_Model lexmark_x73_model = {
@@ -797,6 +808,55 @@ static GT68xx_Model plustek_op1248u_model = {
   /* tested */
 };
 
+static GT68xx_Model plustek_u16b_model = {
+  "plustek-u16b",		/* Name */
+  "Plustek",			/* Device vendor string */
+  "OpticPro U16B",		/* Device model name */
+  "ccd68861.fw",		/* Name of the firmware file */
+  SANE_FALSE,			/* Dynamic allocation flag */
+
+  &plustek_gt6801_command_set,	/* Command set used by this scanner */
+
+  600,				/* maximum optical sensor resolution */
+  1200,				/* maximum motor resolution */
+  600,				/* base x-res used to calculate geometry */
+  600,				/* base y-res used to calculate geometry */
+  50,				/* if ydpi is equal or higher, use linemode */
+  SANE_TRUE,			/* Use base_ydpi for all resolutions */
+
+  {600, 300, 150, 75, 0},	/* possible x-resolutions */
+  {1200, 600, 300, 150, 75, 0},	/* possible y-resolutions */
+  {12, 8, 0},			/* possible depths in gray mode */
+  {12, 8, 0},			/* possible depths in color mode */
+
+  SANE_FIX (3.5),		/* Start of scan area in mm  (x) */
+  SANE_FIX (7.5),		/* Start of scan area in mm (y) */
+  SANE_FIX (218.0),		/* Size of scan area in mm (x) */
+  SANE_FIX (299.0),		/* Size of scan area in mm (y) */
+
+  SANE_FIX (0.0),		/* Start of white strip in mm (y) */
+  SANE_FIX (1.0),		/* Start of black mark in mm (x) */
+
+  SANE_FIX (0.0),		/* Start of scan area in TA mode in mm (x) */
+  SANE_FIX (0.0),		/* Start of scan area in TA mode in mm (y) */
+  SANE_FIX (100.0),		/* Size of scan area in TA mode in mm (x) */
+  SANE_FIX (100.0),		/* Size of scan area in TA mode in mm (y) */
+
+  SANE_FIX (0.0),		/* Start of white strip in TA mode in mm (y) */
+
+  0, 8, 16,			/* RGB CCD Line-distance correction in pixel */
+  0,				/* CCD distcance for CCD with 6 lines) */
+
+  COLOR_ORDER_BGR,		/* Order of the CCD/CIS colors */
+  {0x1c, 0x29, 0x1c, 0x2c, 0x1c, 0x2b},	/* Default offset/gain */
+  {0x157, 0x157, 0x157},	/* Default exposure parameters */
+  SANE_FIX (2.0),		/* Default gamma value */
+
+  SANE_FALSE,			/* Is this a CIS scanner? */
+  GT68XX_FLAG_UNTESTED | GT68XX_FLAG_OFFSET_INV	/* Which flags are needed for this scanner? */
+  /* completely untested */
+};
+
 static GT68xx_Model genius_vivid3x_model  = {
   "genius-colorpage-vivid3x",	/* Name */
   "Genius",			/* Device vendor string */
@@ -842,8 +902,57 @@ static GT68xx_Model genius_vivid3x_model  = {
   SANE_FIX (2.0),		/* Default gamma value */
 
   SANE_FALSE,			/* Is this a CIS scanner? */
-  GT68XX_FLAG_OFFSET_INV |  GT68XX_FLAG_UNTESTED /* Which flags are needed for this scanner? */
-  /* completely untested, based on the Plustek OpticPro 1248U*/
+  GT68XX_FLAG_OFFSET_INV 	/* Which flags are needed for this scanner? */
+  /* Tested zo some degree, based on the Plustek OpticPro 1248U*/
+};
+
+static GT68xx_Model genius_vivid3xe_model  = {
+  "genius-colorpage-vivid3xe",	/* Name */
+  "Genius",			/* Device vendor string */
+  "Colorpage Vivid3xe",		/* Device model name */
+  "ccd548.fw",			/* Name of the firmware file */
+  SANE_FALSE,			/* Dynamic allocation flag */
+
+  &plustek_gt6801_command_set,	/* Command set used by this scanner */
+
+  600,				/* maximum optical sensor resolution */
+  600,				/* maximum motor resolution */
+  600,				/* base x-res used to calculate geometry */
+  600,				/* base y-res used to calculate geometry */
+  50,				/* if ydpi is equal or higher, use linemode */
+  SANE_TRUE,			/* Use base_ydpi for all resolutions */
+
+  {600, 300, 150, 75, 0},	/* possible x-resolutions */
+  {600, 300, 150, 75, 0},	/* possible y-resolutions */
+  {12, 8, 0},			/* possible depths in gray mode */
+  {12, 8, 0},			/* possible depths in color mode */
+
+  SANE_FIX (3.5),		/* Start of scan area in mm  (x) */
+  SANE_FIX (7.5),		/* Start of scan area in mm (y) */
+  SANE_FIX (218.0),		/* Size of scan area in mm (x) */
+  SANE_FIX (299.0),		/* Size of scan area in mm (y) */
+
+  SANE_FIX (0.0),		/* Start of white strip in mm (y) */
+  SANE_FIX (1.0),		/* Start of black mark in mm (x) */
+
+  SANE_FIX (0.0),		/* Start of scan area in TA mode in mm (x) */
+  SANE_FIX (0.0),		/* Start of scan area in TA mode in mm (y) */
+  SANE_FIX (100.0),		/* Size of scan area in TA mode in mm (x) */
+  SANE_FIX (100.0),		/* Size of scan area in TA mode in mm (y) */
+
+  SANE_FIX (0.0),		/* Start of white strip in TA mode in mm (y) */
+
+  0, 8, 16,			/* RGB CCD Line-distance correction in pixel */
+  0,				/* CCD distcance for CCD with 6 lines) */
+
+  COLOR_ORDER_BGR,		/* Order of the CCD/CIS colors */
+  {0x1c, 0x29, 0x1c, 0x2c, 0x1c, 0x2b},	/* Default offset/gain */
+  {0x157, 0x157, 0x157},	/* Default exposure parameters */
+  SANE_FIX (2.0),		/* Default gamma value */
+
+  SANE_FALSE,			/* Is this a CIS scanner? */
+  GT68XX_FLAG_OFFSET_INV /* Which flags are needed for this scanner? */
+  /* mostly untested, based on the Genius Vivid3x */
 };
 
 static GT68xx_USB_Device_Entry gt68xx_usb_device_list[] = {
@@ -859,7 +968,10 @@ static GT68xx_USB_Device_Entry gt68xx_usb_device_list[] = {
   {0x05d8, 0x4002, &mustek_scanexpress2400usb_model}, /* manual override */
   {0x055f, 0x0210, &mustek_a3usb_model},
   {0x043d, 0x002d, &lexmark_x73_model},
-  {0x07b3, 0x0401, &plustek_op1248u_model},
+  {0x07b3, 0x0400, &plustek_op1248u_model},
+  {0x07b3, 0x0401, &plustek_op1248u_model}, /* Same scanner, different id? */
+  {0x07b3, 0x0403, &plustek_u16b_model},
   {0x0458, 0x2011, &genius_vivid3x_model},
+  {0x0458, 0x2017, &genius_vivid3xe_model},
   {0, 0, NULL}
 };
