@@ -1,7 +1,7 @@
 /* sane - Scanner Access Now Easy.
 
    Copyright (C) 2002 Sergey Vlasov <vsu@altlinux.ru>
-   Copyright (C) 2002 Henning Meier-Geinitz <henning@meier-geinitz.de>
+   Copyright (C) 2002, 2003 Henning Meier-Geinitz <henning@meier-geinitz.de>
    
    This file is part of the SANE package.
    
@@ -499,9 +499,10 @@ gt68xx_device_memory_read (GT68xx_Device * dev,
 
 static SANE_Status
 gt68xx_device_generic_req (GT68xx_Device * dev,
+			   SANE_Byte request_type, SANE_Word request,
 			   SANE_Word cmd_value, SANE_Word cmd_index,
 			   SANE_Word res_value, SANE_Word res_index,
-			   GT68xx_Packet cmd, GT68xx_Packet res)
+			   GT68xx_Packet cmd, GT68xx_Packet res, size_t res_size)
 {
   SANE_Status status;
 
@@ -510,7 +511,7 @@ gt68xx_device_generic_req (GT68xx_Device * dev,
   CHECK_DEV_ACTIVE (dev, "gt68xx_device_generic_req");
 
   status = sanei_usb_control_msg (dev->fd,
-				  0x40, 0x01, cmd_value, cmd_index,
+				  request_type, request, cmd_value, cmd_index,
 				  GT68XX_PACKET_SIZE, cmd);
   if (status != SANE_STATUS_GOOD)
     {
@@ -522,8 +523,8 @@ gt68xx_device_generic_req (GT68xx_Device * dev,
   memset (res, 0, sizeof (GT68xx_Packet));
 
   status = sanei_usb_control_msg (dev->fd,
-				  0xc0, 0x01, res_value, res_index,
-				  GT68XX_PACKET_SIZE, res);
+				  request_type | 0x80, request, res_value, res_index,
+				  res_size, res);
   if (status != SANE_STATUS_GOOD)
     {
       DBG (3, "gt68xx_device_generic_req: reading response failed: %s\n",
@@ -540,10 +541,13 @@ gt68xx_device_req (GT68xx_Device * dev, GT68xx_Packet cmd, GT68xx_Packet res)
 {
   GT68xx_Command_Set *command_set = dev->model->command_set;
   return gt68xx_device_generic_req (dev,
+				    command_set->request_type,
+				    command_set->request,
 				    command_set->send_cmd_value,
 				    command_set->send_cmd_index,
 				    command_set->recv_res_value,
-				    command_set->recv_res_index, cmd, res);
+				    command_set->recv_res_index, cmd, res,
+				    GT68XX_PACKET_SIZE);
 }
 
 SANE_Status
@@ -558,11 +562,13 @@ gt68xx_device_small_req (GT68xx_Device * dev, GT68xx_Packet cmd,
     memcpy (fixed_cmd + i * 8, cmd, 8);
 
   return gt68xx_device_generic_req (dev,
+				    command_set->request_type,
+				    command_set->request,
 				    command_set->send_small_cmd_value,
 				    command_set->send_small_cmd_index,
 				    command_set->recv_small_res_value,
 				    command_set->recv_small_res_index,
-				    fixed_cmd, res);
+				    fixed_cmd, res, 0x08);
 }
 
 SANE_Status
