@@ -7,7 +7,7 @@
  *  @brief Here we have our USB device definitions.
  *
  * Based on sources acquired from Plustek Inc.<br>
- * Copyright (C) 2001-2002 Gerhard Jaeger <gerhard@gjaeger.de>
+ * Copyright (C) 2001-2003 Gerhard Jaeger <gerhard@gjaeger.de>
  *
  * History:
  * - 0.40 - starting version of the USB support
@@ -37,6 +37,7 @@
  *        - tweaked EPSON 1260 settings
  *        - removed EPSON 660 stuff
  *        - added Canon 1220U entry
+ *        - added entry for Compaq S4-100
  * .
  * <hr>
  * This file is part of the SANE package.
@@ -471,17 +472,17 @@ static DCapsDef Cap0x07B3_0x0017_3 =
 	8, 4, kNEC8861,	0x07, _WAF_NONE, _NO_MIO				
 };
 
-/* Model: HP Scanjet 2100C */
+/* Model: HP Scanjet 2100c */
 static DCapsDef Cap0x03F0_0x0505 =
 {
-	{{ 0, 208}, 0, -1, {2550, 3508}, { 50,  50}, COLOR_BW },
-    {{ 0,   0}, 0, -1, {0, 0}, { 0, 0 }, 0 }, /* No film scanner module    */
-    {{ 0,   0}, 0, -1, {0, 0}, { 0, 0 }, 0 }, /* No film scanner module    */
-    {{ 0,   0}, 0, -1, {0, 0}, { 0, 0 }, 0 }, /* No ADF                    */
+	{{ 0,  65}, 10, -1, {2550, 3508}, { 50,  50}, COLOR_BW },
+    {{ 0,   0},  0, -1, {0, 0}, { 0, 0 }, 0 }, /* No film scanner module    */
+    {{ 0,   0},  0, -1, {0, 0}, { 0, 0 }, 0 }, /* No film scanner module    */
+    {{ 0,   0},  0, -1, {0, 0}, { 0, 0 }, 0 }, /* No ADF                    */
  	{600, 600},
  	0,
  	SENSORORDER_rgb,
- 	4, 0, kNEC3799, 0x02, _WAF_NONE, _NO_MIO					
+ 	4, 0, kNECSLIM, 0x00, _WAF_NONE, _NO_MIO					
 };
 
 /* Model: HP Scanjet 2200c (thanks to Stefan Nilsen)
@@ -490,7 +491,7 @@ static DCapsDef Cap0x03F0_0x0505 =
 static DCapsDef Cap0x03F0_0x0605 =
 {
 	/* DataOrigin (x, y), ShadingOriginY */
-	{{ 0, 209},  0, -1, {2550, 3508 }, { 50,  50}, COLOR_BW },
+	{{ 0, 209},  0, -1, {2550, 3508}, { 50,  50}, COLOR_BW },
     {{ 0,   0},  0, -1, {0, 0}, { 0, 0 }, 0 }, /* No film scanner module    */
     {{ 0,   0},  0, -1, {0, 0}, { 0, 0 }, 0 }, /* No film scanner module    */
     {{ 0,   0},  0, -1, {0, 0}, { 0, 0 }, 0 }, /* No ADF                    */
@@ -587,16 +588,16 @@ static DCapsDef Cap0x1606_0x0060_0 =
  */
 static DCapsDef Cap0x1606_0x0160_0 =
 {
- 	{{ 0, 165}, 0, -1, {2550, 3510 - 165}, {100, 100}, COLOR_BW },
- 	{{ 0,  0},  0, -1, {0, 0}, { 0, 0 }, 0 },
- 	{{ 0,  0},  0, -1, {0, 0}, { 0, 0 }, 0 },
- 	{{ 0,  0},  0, -1, {0, 0}, { 0, 0 }, 0 },
+ 	{{ 30, 165}, 0, -1, {2550, 3508}, {100, 100}, COLOR_BW },
+ 	{{  0,   0}, 0, -1, {0, 0}, { 0, 0 }, 0 },
+ 	{{  0,   0}, 0, -1, {0, 0}, { 0, 0 }, 0 },
+ 	{{  0,   0}, 0, -1, {0, 0}, { 0, 0 }, 0 },
  	{1200, 1200},
  	0,
  	SENSORORDER_bgr,
  	12,			        /* sensor distance                         */
  	4,		      	    /* number of buttons                       */
- 	kNEC8861,           /* use default settings during calibration */
+ 	kNEC3778,           /* use default settings during calibration */
  	0,                  /* not used here...                        */
  	_WAF_MISC_IO_LAMPS, /* use miscio 3 for lamp switching         */
     _MIO3
@@ -606,7 +607,7 @@ static DCapsDef Cap0x1606_0x0160_0 =
  */
 static DCapsDef Cap0x04A9_0x2206_0 =
 {
- 	{{ 0, 90}, 35, -1, {2550, 3508}, {75, 75}, COLOR_GRAY16 },
+ 	{{ 0, 90}, 35, 10, {2550, 3508}, {75, 75}, COLOR_GRAY16 },
  	{{ 0,  0},  0,  0, {0, 0}, { 0, 0 }, 0 },
  	{{ 0,  0},  0,  0, {0, 0}, { 0, 0 }, 0 },
  	{{ 0,  0},  0,  0, {0, 0}, { 0, 0 }, 0 },
@@ -615,16 +616,16 @@ static DCapsDef Cap0x04A9_0x2206_0 =
  	SENSORORDER_rgb,
  	8,			        /* sensor distance                         */
  	1,		      	    /* number of buttons                       */
- 	kNEC8861,           /* use default settings during calibration */
+ 	kCIS650,            /* use default settings during calibration */
  	0,                  /* not used here...                        */
-	(_WAF_MISC_IO_LAMPS | _WAF_SKIP_WHITEFINE), _NO_MIO
+    _WAF_MISC_IO_LAMPS | _WAF_BLACKFINE | _WAF_FIX_OFS, _NO_MIO
 };
 
 /* Canon N1220U
  */
 static DCapsDef Cap0x04A9_0x2207_0 =
 {
-	{{ 0, 85}, 35, -1, {2550, 3508}, {75, 75}, COLOR_BW },
+	{{ 0, 85}, 35, 10, {2550, 3508}, {75, 75}, COLOR_BW },
     {{ 0,  0},  0,  0, {0, 0}, { 0, 0 }, 0 },
     {{ 0,  0},  0,  0, {0, 0}, { 0, 0 }, 0 },
     {{ 0,  0},  0,  0, {0, 0}, { 0, 0 }, 0 },
@@ -633,16 +634,16 @@ static DCapsDef Cap0x04A9_0x2207_0 =
     SENSORORDER_rgb,
     16,                 /* sensor distance                         */
     1,                  /* number of buttons                       */
-    kNEC8861,           /* use default settings during calibration */
+    kCIS650,            /* use default settings during calibration */
     0,                  /* not used here...                        */
-    (_WAF_MISC_IO_LAMPS | _WAF_SKIP_WHITEFINE), _NO_MIO
+    _WAF_MISC_IO_LAMPS | _WAF_BLACKFINE | _WAF_FIX_OFS, _NO_MIO
 };
 
 /* Canon N670U/N676U/LiDE20
  */
 static DCapsDef Cap0x04A9_0x220D_0 =
 {
- 	{{ 0, 100}, 35, -1, {2550, 3508}, {75, 75}, COLOR_GRAY16 },
+ 	{{ 0, 110}, 35, 10, {2550, 3508}, {75, 75}, COLOR_GRAY16 },
  	{{ 0,   0},  0,  0, {0, 0}, { 0, 0 }, 0 },
  	{{ 0,   0},  0,  0, {0, 0}, { 0, 0 }, 0 },
  	{{ 0,   0},  0,  0, {0, 0}, { 0, 0 }, 0 },
@@ -651,16 +652,16 @@ static DCapsDef Cap0x04A9_0x220D_0 =
  	SENSORORDER_rgb,
  	8,			        /* sensor distance                         */
  	3,		      	    /* number of buttons                       */
- 	kNEC8861,           /* use default settings during calibration */
+ 	kCIS670,            
  	0,                  /* not used here...                        */
-	(_WAF_MISC_IO_LAMPS | _WAF_SKIP_WHITEFINE), _NO_MIO
+	_WAF_MISC_IO_LAMPS | _WAF_BLACKFINE | _WAF_FIX_OFS, _NO_MIO
 };
 
 /* Canon N1240U
  */
 static DCapsDef Cap0x04A9_0x220E_0 =
 {
-	{{ 0, 100}, 35, -1, {2550, 3508}, {75, 75}, COLOR_BW },
+	{{ 0, 100}, 50, 10, {2550, 3508}, {75, 75}, COLOR_BW },
     {{ 0,   0},  0,  0, {0, 0}, { 0, 0 }, 0 },
     {{ 0,   0},  0,  0, {0, 0}, { 0, 0 }, 0 },
     {{ 0,   0},  0,  0, {0, 0}, { 0, 0 }, 0 },
@@ -669,9 +670,9 @@ static DCapsDef Cap0x04A9_0x220E_0 =
     SENSORORDER_rgb,
     16,                 /* sensor distance                         */
     3,                  /* number of buttons                       */
-    kNEC8861,           /* use default settings during calibration */
+    kCIS1240,           /* use default settings during calibration */
     0,                  /* not used here...                        */
-    (_WAF_MISC_IO_LAMPS | _WAF_SKIP_WHITEFINE), _NO_MIO
+    _WAF_MISC_IO_LAMPS | _WAF_BLACKFINE | _WAF_FIX_OFS, _NO_MIO
 };
 
 /******************* additional Hardware descriptions ************************/
@@ -1328,9 +1329,9 @@ static HWDef Hw0x03F0_0x0505 =
 	0,	    /* wGreenPWMDutyCycleLow                    */
 	0,	    /* wGreenPWMDutyCycleHigh                   */
 	0x02,	/* bSensorConfiguration (0x0b)              */
-	0x04,	/* bReg_0x0c                                */
+	0x00,	/* bReg_0x0c                                */
 	0x2F,	/* bReg_0x0d                                */
-	0x1F,	/* bReg_0x0e                                */
+	0x13,	/* bReg_0x0e                                */
 		    /* bReg_0x0f_Mono[10] (0x0f to 0x18)        */
 
 	{ 0x02, 0x07, 0x01, 0x02, 0x02, 0x03, 0x00, 0x00, 0x04, 0x07 },
@@ -1348,10 +1349,10 @@ static HWDef Hw0x03F0_0x0505 =
 	{ 0, 0, 0, 0, 0, 0, 0 },
 
 	1,		/* StepperPhaseCorrection (0x1a & 0x1b)     */
-	14,		/* bOpticBlackStart (0x1c)             		*/
-	63,		/* bOpticBlackEnd (0x1d)               		*/
+	15,		/* bOpticBlackStart (0x1c)             		*/
+	50,		/* bOpticBlackEnd (0x1d)               		*/
 	140,	/* wActivePixelsStart (0x1e & 0x1f)    		*/
-	5367, 	/* wLineEnd=(0x20 & 0x21)       			*/
+	5414, 	/* wLineEnd=(0x20 & 0x21)       			*/
 
     1,      /* red lamp on    (reg 0x2c + 0x2d)         */
     16383,  /* red lamp off   (reg 0x2e + 0x2f)         */
@@ -1370,8 +1371,8 @@ static HWDef Hw0x03F0_0x0505 =
 	8,		/* bReg_0x56                                */
 	60,		/* bReg_0x57                                */
 	0x0d,	/* bReg_0x58                                */
-	0xcc,	/* bReg_0x59                                */
-	0xbc,	/* bReg_0x5a                                */
+	0xaa,	/* bReg_0x59                                */
+	0xba,	/* bReg_0x5a                                */
 	0xbb,	/* bReg_0x5b                                */
 	0,		/* bReg_0x5c                                */
 	0,		/* bReg_0x5d                                */
@@ -1761,8 +1762,8 @@ static HWDef Hw0x1606_0x0060_0 =
 /** Umax 5400 */
 static HWDef Hw0x1606_0x0160_0 =
 {
-    1.7,    /* dMaxMotorSpeed (Max_Speed)                */
-    0.8,    /* dMaxMoveSpeed (Max_Speed)                 */
+    1.1,    /* dMaxMotorSpeed (Max_Speed)                */
+    0.9,    /* dMaxMoveSpeed (Max_Speed)                 */
     9,      /* dIntegrationTimeLowLamp                   */
     9,      /* dIntegrationTimeHighLamp                  */
     600,    /* wMotorDpi (Full step DPI)                 */
@@ -1819,8 +1820,8 @@ static HWDef Hw0x1606_0x0160_0 =
     0,      /* test mode ADC Output CODE MSB (reg 0x5c)  */
     0,      /* test mode ADC Output CODE LSB (reg 0x5d)  */
     0,      /* test mode (reg 0x5e)                      */
-    _LM9832, /* might be LM9831 on UMAX 3450! */
-	MODEL_UMAX
+    _LM9832,
+	MODEL_UMAX1200
 };
 
 /** Canon 650/656 */
@@ -1862,8 +1863,6 @@ static HWDef Hw0x04A9_0x2206_0 =
     89,     /* ? wActivePixelsStart (reg 0x1e + 0x1f)         */
     6074,   /* wLineEnd (reg 0x20 + 0x21)                     */
 
-/* 0x17ba = 6074 bis 100dpi, 0x14ba = 5306 */
-
     23,  	/* red lamp on    (reg 0x2c + 0x2d)               */
     2416,   /* red lamp off   (reg 0x2e + 0x2f)               */
     23,  	/* green lamp on  (reg 0x30 + 0x31)               */
@@ -1880,14 +1879,14 @@ static HWDef Hw0x04A9_0x2206_0 =
     0x08,   /* pwm freq (reg 0x56)                            */
     0x1f,   /* pwm duty cycle (reg 0x57)                      */
 
-    0x05,   /* Paper sense (reg 0x58)                    */
+    0x05,   /* Paper sense (reg 0x58)                         */
 
-    0x24,   /* misc io12 (reg 0x59)                      */
-    0x96,   /* misc io34 (reg 0x5a)                      */
-    0xb9,   /* misc io56 (reg 0x5b)                      */
-    0x01,   /* test mode ADC Output CODE MSB (reg 0x5c)  */
-    0,      /* test mode ADC Output CODE LSB (reg 0x5d)  */
-    0,      /* test mode (reg 0x5e)                      */
+    0x66,   /* misc io12 (reg 0x59)                           */
+    0x16,   /* misc io34 (reg 0x5a)                           */
+    0x91,   /* misc io56 (reg 0x5b)                           */
+    0x01,   /* test mode ADC Output CODE MSB (reg 0x5c)       */
+    0,      /* test mode ADC Output CODE LSB (reg 0x5d)       */
+    0,      /* test mode (reg 0x5e)                           */
     _LM9832,
 	MODEL_CANON600
 };
@@ -1922,7 +1921,7 @@ static HWDef Hw0x04A9_0x2207_0 =
     0x00,   /* bReg 0x27 color mode                           */
     2,      /* bReg 0x29 illumination mode                    */
 	{ 3,  0,     0, 23, 3937,  0,    0 },
-	{ 2, 23, 12000, 23, 5500, 23, 3900 },
+	{ 2, 23, 14000, 23, 7500, 23, 5900 },
 
     1,      /* StepperPhaseCorrection (reg 0x1a + 0x1b)       */
     0,      /* bOpticBlackStart (reg 0x1c)                    */
@@ -1948,9 +1947,9 @@ static HWDef Hw0x04A9_0x2207_0 =
 
     0x05,   /* Paper sense (reg 0x58)                         */
 
-    0x44,   /* misc io12 (reg 0x59)                           */
-    0x94,   /* misc io34 (reg 0x5a)                           */
-    0x19,   /* misc io56 (reg 0x5b)                           */
+    0x66,   /* misc io12 (reg 0x59)                           */
+    0x16,   /* misc io34 (reg 0x5a)                           */
+    0x91,   /* misc io56 (reg 0x5b)                           */
     0x01,   /* test mode ADC Output CODE MSB (reg 0x5c)       */
     0,      /* test mode ADC Output CODE LSB (reg 0x5d)       */
     0,      /* test mode (reg 0x5e)                           */
@@ -1989,7 +1988,7 @@ static HWDef Hw0x04A9_0x220D_0 =
 	2,      /* bReg 0x29 illumination mode (runtime)          */
 
 	{ 3,  0,    0, 23, 1800,  0,    0 },
-	{ 2, 23, 3562, 23, 3315, 23, 2676 },
+	{ 2, 23, 3800, 23, 3300, 23, 2700 },
 
     1,      /* StepperPhaseCorrection (reg 0x1a + 0x1b)       */
     0,      /* bOpticBlackStart (reg 0x1c)                    */
@@ -2017,9 +2016,9 @@ static HWDef Hw0x04A9_0x220D_0 =
 
     0x04,   /* Paper sense (reg 0x58)                         */
 
-    0x44,   /* misc io12 (reg 0x59)                           */
-    0x94,   /* misc io34 (reg 0x5a)                           */
-    0x19,   /* misc io56 (reg 0x5b)                           */
+    0x66,   /* misc io12 (reg 0x59)                           */
+    0x16,   /* misc io34 (reg 0x5a)                           */
+    0x91,   /* misc io56 (reg 0x5b)                           */
     0x01,   /* test mode ADC Output CODE MSB (reg 0x5c)       */
     0,      /* test mode ADC Output CODE LSB (reg 0x5d)       */
     0,      /* test mode (reg 0x5e)                           */
@@ -2056,8 +2055,13 @@ static HWDef Hw0x04A9_0x220E_0 =
 
     0x00,   /* bReg 0x27 color mode                           */
     2,      /* bReg 0x29 illumination mode                    */
-	{ 3,  0,     0, 23, 3937,  0,    0 },
-	{ 2, 23, 12000, 23, 5500, 23, 3900 },
+#if 1    
+    { 3,  0,     0, 23, 3937,  0,    0 },
+    { 2, 23, 13000, 23, 6500, 23, 4900 },
+#else	
+	{ 3,  0,     0, 23, 16383,  0,     0 },
+	{ 2, 23, 16383, 23, 16383, 23, 16383 },
+#endif
 
     1,      /* StepperPhaseCorrection (reg 0x1a + 0x1b)       */
     0,      /* bOpticBlackStart (reg 0x1c)                    */
@@ -2085,9 +2089,9 @@ static HWDef Hw0x04A9_0x220E_0 =
 
     0x04,   /* Paper sense (reg 0x58)                         */
 
-    0x44,   /* misc io12 (reg 0x59)                           */
-    0x94,   /* misc io34 (reg 0x5a)                           */
-    0x19,   /* misc io56 (reg 0x5b)                           */
+    0x66,   /* misc io12 (reg 0x59)                           */
+    0x16,   /* misc io34 (reg 0x5a)                           */
+    0x91,   /* misc io56 (reg 0x5b)                           */
     0x01,   /* test mode ADC Output CODE MSB (reg 0x5c)       */
     0,      /* test mode ADC Output CODE LSB (reg 0x5d)       */
     0,      /* test mode (reg 0x5e)                           */
@@ -2152,7 +2156,10 @@ static SetDef Settings[] =
 	/* UMAX... */
  	{"0x1606-0x0060",	&Cap0x1606_0x0060_0, &Hw0x1606_0x0060_0, "3400/3450" },
  	{"0x1606-0x0160",	&Cap0x1606_0x0160_0, &Hw0x1606_0x0160_0, "5400"      },
-		
+  
+	/* COMPAQ... */
+ 	{"0x049F-0x001A",	&Cap0x1606_0x0060_0, &Hw0x1606_0x0060_0, "S4-100" },
+
 	/* CANON... */
 	{"0x04A9-0x2206",   &Cap0x04A9_0x2206_0, &Hw0x04A9_0x2206_0, "N650U/N656U" },
 	{"0x04A9-0x2207",   &Cap0x04A9_0x2207_0, &Hw0x04A9_0x2207_0, "N1220U"      },
@@ -2261,6 +2268,7 @@ static ClkMotorDef Motors[] = {
 		/* Motor settings (PWM and PWM_Duty) */
 	    {{ 8, 60, 1 }, { 8, 60, 1 }, { 8, 60, 1 }, { 8, 60, 1 }, { 8, 60, 1 },
 		 { 8, 60, 1 }, { 8, 60, 1 }, { 8, 60, 1 }, { 8, 60, 1 }, { 8, 60, 1 }},
+
         /* Color mode MCLK settings */
 	    { 4.0, 4.0, 4.0, 4.0, 3.0, 4.0, 6.0, 6.0, 6.0, 6.0 },
 	    { 4.0, 4.0, 4.0, 4.0, 3.0, 4.0, 6.0, 6.0, 6.0, 6.0 },
@@ -2314,6 +2322,18 @@ static ClkMotorDef Motors[] = {
 	    { 10.5, 10.5, 10.5, 10.5, 10.5, 10.5, 10.5, 10.5, 10.5, 10.5 }
 	},
 
+	{ MODEL_UMAX1200, 16, 4, 6,
+		/* Motor settings (PWM and PWM_Duty) */
+	    {{ 16, 4, 1 }, { 16, 4, 1 }, { 16, 4, 1 }, { 16, 4, 1 }, { 16, 4, 1 },
+		 { 16, 4, 1 }, { 16, 4, 1 }, { 16, 4, 1 }, { 16, 4, 1 }, { 16, 4, 1 }},
+        /* Color mode MCLK settings */
+	    { 3.0, 3.0, 3.0, 3.0, 3.0, 6.0, 6.0, 6.0, 6.0, 6.0 },
+	    { 5.0, 5.0, 5.0, 5.0, 5.0, 5.0, 5.0, 5.0, 5.0, 5.0 },
+		/* Gray mode MCLK settings */
+	    {  6.0,  6.0,  6.0,  6.0,  6.0, 13.0, 13.0, 13.0, 13.0, 13.0 },
+	    { 13.0, 13.0, 13.0, 13.0, 13.0, 13.0, 13.0, 13.0, 13.0, 13.0 }
+	},
+
 	/* settings good for the EPSON models (tested with 1260) */
 	{ MODEL_EPSON, 2, 1, 6,
 		/* Motor settings (PWM and PWM_Duty) */
