@@ -7,7 +7,7 @@
  *  @brief Image processing functions for copying and scaling image lines.
  *
  * Based on sources acquired from Plustek Inc.<br>
- * Copyright (C) 2001-2002 Gerhard Jaeger <gerhard@gjaeger.de>
+ * Copyright (C) 2001-2003 Gerhard Jaeger <gerhard@gjaeger.de>
  *
  * History:
  * - 0.40 - starting version of the USB support
@@ -18,6 +18,10 @@
  * - 0.45 - added gray scaling functions for CIS devices
  *        - fixed usb_GrayScale16 function
  *        - fixed a bug in usb_ColorScale16_2 function
+ *        - fixed endless loop bug
+ *        - fixed a bug in usb_GrayScalePseudo16 function
+ *        - fixed a bug in usb_GrayDuplicatePseudo16 function
+ *        - removed the scaler stuff for CIS devices
  * .
  * <hr>
  * This file is part of the SANE package.
@@ -709,7 +713,7 @@ static void usb_ColorScale16_2( struct Plustek_Device *dev )
 	}
 }
 
-/*.............................................................................
+/**
  *
  */
 static void usb_ColorScalePseudo16( struct Plustek_Device *dev )
@@ -911,7 +915,7 @@ static void usb_ColorDuplicate16_2( struct Plustek_Device *dev )
 	}
 }
 
-/*.............................................................................
+/**
  *
  */
 static void usb_ColorDuplicatePseudo16( struct Plustek_Device *dev )
@@ -938,9 +942,13 @@ static void usb_ColorDuplicatePseudo16( struct Plustek_Device *dev )
 
 	for (dw = 0; dw < scanning->sParam.Size.dwPixels; dw++, dwPixels = dwPixels + iNext)
 	{
-		scanning->UserBuf.pw_rgb[dwPixels].Red = (wR + scanning->Red.pcb[dw].a_bColor[0]) << bShift;
-		scanning->UserBuf.pw_rgb[dwPixels].Green = (wG + scanning->Green.pcb[dw].a_bColor[0]) << bShift;
-		scanning->UserBuf.pw_rgb[dwPixels].Blue = (wB + scanning->Blue.pcb[dw].a_bColor[0]) << bShift;
+		scanning->UserBuf.pw_rgb[dwPixels].Red   =
+						(wR + scanning->Red.pcb[dw].a_bColor[0])   << bShift;
+		scanning->UserBuf.pw_rgb[dwPixels].Green =
+						(wG + scanning->Green.pcb[dw].a_bColor[0]) << bShift;
+		scanning->UserBuf.pw_rgb[dwPixels].Blue  =
+						(wB + scanning->Blue.pcb[dw].a_bColor[0])  << bShift;
+
 		wR = (u_short)scanning->Red.pcb[dw].a_bColor[0];
 		wG = (u_short)scanning->Green.pcb[dw].a_bColor[0];
 		wB = (u_short)scanning->Blue.pcb[dw].a_bColor[0];
@@ -1111,9 +1119,10 @@ static void usb_GrayScale8( struct Plustek_Device *dev )
 		iNext = 1;
 	}
 	
-	izoom = usb_GetScaler( scanning );			
+	izoom = usb_GetScaler( scanning );
+	ddax  = 0;
 
-	for( dwPixels = scanning->sParam.Size.dwPixels, ddax = 0; dwPixels; pbSrce++ ) {
+	for( dwPixels = scanning->sParam.Size.dwPixels; dwPixels; pbSrce++ ) {
 
 		ddax -= _SCALER;
 
@@ -1193,9 +1202,6 @@ static void usb_GrayScalePseudo16( struct Plustek_Device *dev )
 
 	usb_AverageGrayByte( dev );
 
-	pbSrce = scanning->Green.pb;
-	wSum = scanning->sParam.PhyDpi.x;
-	wG = (u_short) *pbSrce;
 	if( scanning->sParam.bSource == SOURCE_ADF ) {
 		iNext  = -1;
 		pwDest = scanning->UserBuf.pw + scanning->sParam.Size.dwPixels - 1;
@@ -1204,17 +1210,20 @@ static void usb_GrayScalePseudo16( struct Plustek_Device *dev )
 		pwDest = scanning->UserBuf.pw;
 	}
 
+	pbSrce = scanning->Green.pb;
+	wG     = (u_short)*pbSrce;
+
 	izoom = usb_GetScaler( scanning );			
 	ddax  = 0;
 
-	for( dwPixels = scanning->sParam.Size.dwPixels, ddax = 0; dwPixels; pbSrce++ ) {
+	for( dwPixels = scanning->sParam.Size.dwPixels; dwPixels; pbSrce++ ) {
 
 		ddax -= _SCALER;
 
 		while((ddax < 0) && (dwPixels > 0)) {
 
 			*pwDest = (wG + *pbSrce) << bShift;
-			pbDest  = pbDest + iNext;
+			pwDest  = pwDest + iNext;
 			ddax   += izoom;
 			dwPixels--;
 		} 			
@@ -1282,21 +1291,23 @@ static void usb_GrayDuplicatePseudo16( struct Plustek_Device *dev )
 
 	usb_AverageGrayByte( dev );
 
-	pbSrce = scanning->UserBuf.pb;
 	if (scanning->sParam.bSource == SOURCE_ADF)
 	{
 		iNext = -1;
-		pwDest = scanning->Green.pw + scanning->sParam.Size.dwPixels - 1;
+		pwDest = scanning->UserBuf.pw + scanning->sParam.Size.dwPixels - 1;
 	}
 	else
 	{
 		iNext = 1;
-		pwDest = scanning->Green.pw;
+		pwDest = scanning->UserBuf.pw;
 	}
+
+	pbSrce = scanning->Green.pb;
 	wG = (u_short)*pbSrce;
 
 	for( dwPixels = scanning->sParam.Size.dwPixels;
-		 dwPixels++; pbSrce++, pwDest = pwDest + iNext ) {
+							 dwPixels--; pbSrce++, pwDest = pwDest + iNext ) {
+
 		*pwDest = (wG + *pbSrce) << bShift;
 		wG = (u_short)*pbSrce;
 	}
@@ -1471,76 +1482,69 @@ static void usb_GetImageProc( struct Plustek_Device *dev )
  */
 static SANE_Int usb_ReadData( struct Plustek_Device *dev )
 {
-	u_long   dw, dwRet, dwBytes, dwAdjust;
+	u_long   dw, dwRet, dwBytes, pl;
     pScanDef scanning = &dev->scanning;
 	pHWDef   hw       = &dev->usbDev.HwSetting;
 
 	DBG( _DBG_READ, "usb_ReadData()\n" );
 
-	dwAdjust = 1;                                                
-
-	/* for 1 channel color, we have to adjust the phybytes... */
-	if( hw->bReg_0x26 & _ONE_CH_COLOR ) {
-		if( scanning->sParam.bDataType == SCANDATATYPE_Color ) {
-			dwAdjust = 3;
-		}
-	}
+	pl = a_bRegs[0x4e] * hw->wDRAMSize/128;
 
 	while( scanning->sParam.Size.dwTotalBytes ) {
-	
+
 		if( usb_IsEscPressed()) {
 			DBG( _DBG_INFO, "usb_ReadData() - Cancel detected...\n" );
 			return 0;
 		}
-   		
+
    		if( scanning->sParam.Size.dwTotalBytes > scanning->dwBytesScanBuf )
    			dw = scanning->dwBytesScanBuf;
    		else
    			dw = scanning->sParam.Size.dwTotalBytes;
-   			
+
    		scanning->sParam.Size.dwTotalBytes -= dw;
 
-   		if(!scanning->sParam.Size.dwTotalBytes && dw < (m_dwPauseLimit * 1024))
+   		if(!scanning->sParam.Size.dwTotalBytes && dw < (pl * 1024))
    		{
    			if(!(a_bRegs[0x4e] = (u_char)ceil((double)dw /
 													(4.0 * hw->wDRAMSize)))) {
    				a_bRegs[0x4e] = 1;
 			}
-   				
+
    			a_bRegs[0x4f] = 0;
-				
+
    			sanei_lm983x_write( dev->fd, 0x4e, &a_bRegs[0x4e], 2, SANE_TRUE );
    		}
 
    		while( scanning->bLinesToSkip ) {
-			
+
    			DBG( _DBG_READ, "Skipping %u lines\n", scanning->bLinesToSkip );
 
    			dwBytes = scanning->bLinesToSkip*scanning->sParam.Size.dwPhyBytes;
-			dwBytes *= dwAdjust;
-				
+
    			if (dwBytes > scanning->dwBytesScanBuf) {
-				
+
    				dwBytes = scanning->dwBytesScanBuf;
    				scanning->bLinesToSkip -= scanning->dwLinesScanBuf;
    			} else {
    				scanning->bLinesToSkip = 0;
-   			}						
-					
-   			usb_ScanReadImage( dev, scanning->pbGetDataBuf, dwBytes );
+   			}
+
+   			if( !usb_ScanReadImage( dev, scanning->pbGetDataBuf, dwBytes ))
+				return 0;
    		}
-		
+
    	    if( usb_ScanReadImage( dev, scanning->pbGetDataBuf, dw )) {
 
 			dumpPic( "plustek-pic.raw", scanning->pbGetDataBuf, dw );
-		
+
    	    	if( scanning->dwLinesDiscard ) {
 
 	   			DBG( _DBG_READ, "Discarding %lu lines\n",
 			   										scanning->dwLinesDiscard );
-   	    	   	    	
-   	    		dwRet = dw / (scanning->sParam.Size.dwPhyBytes * dwAdjust);
-   	    		
+
+   	    		dwRet = dw / scanning->sParam.Size.dwPhyBytes;
+
    	    		if (scanning->dwLinesDiscard > dwRet) {
    	    			scanning->dwLinesDiscard -= dwRet;
    	    			dwRet = 0;
@@ -1550,19 +1554,19 @@ static SANE_Int usb_ReadData( struct Plustek_Device *dev )
    	    		}
    	    	} else {
 
-   	    		dwRet = dw / (scanning->sParam.Size.dwPhyBytes * dwAdjust);
-   	    	}	
+   	    		dwRet = dw / scanning->sParam.Size.dwPhyBytes;
+   	    	}
 
    			scanning->pbGetDataBuf += scanning->dwBytesScanBuf;
    			if( scanning->pbGetDataBuf >= scanning->pbScanBufEnd ) {
    				scanning->pbGetDataBuf = scanning->pbScanBufBegin;
-   			}	
-   	    	
+   			}
+
    	    	if( dwRet )
    	    		return dwRet;
    	    }
 	}
-	
+
 	return 0;
 }
 
