@@ -46,7 +46,7 @@
    This file is a component of the implementation of a backend for many
    of the the AGFA SnapScan and Acer Vuego/Prisa flatbed scanners. */
 
-/* $Id: snapscan-options.c,v 1.5 2002/07/12 23:29:05 oliverschwartz Exp $
+/* $Id: snapscan-options.c,v 1.7 2003/04/30 20:49:39 oliverschwartz Exp $
    SANE SnapScan backend */
 
 /* ranges */
@@ -211,6 +211,8 @@ static void init_options (SnapScan_Scanner * ps)
     case SNAPSCANE52:
     case PRISA5300:
     case PRISA1240:
+    case PRISA5000:
+    case ARCUS1200:
         po[OPT_SCANRES].constraint.word_list = resolutions_1200;
         break;
     default:
@@ -1386,8 +1388,14 @@ SANE_Status sane_control_option (SANE_Handle h,
 
 /*
  * $Log: snapscan-options.c,v $
- * Revision 1.5  2002/07/12 23:29:05  oliverschwartz
- * SnapScan backend 1.4.15
+ * Revision 1.7  2003/04/30 20:49:39  oliverschwartz
+ * SnapScan backend 1.4.26
+ *
+ * Revision 1.8  2003/04/30 20:42:18  oliverschwartz
+ * Added support for Agfa Arcus 1200 (supplied by Valtteri Vuorikoski)
+ *
+ * Revision 1.7  2003/04/02 21:17:12  oliverschwartz
+ * Fix for 1200 DPI with Acer 5000
  *
  * Revision 1.6  2002/07/12 23:23:06  oliverschwartz
  * Disable quality calibration for 5300
