@@ -47,12 +47,13 @@
    of the the AGFA SnapScan and Acer Vuego/Prisa flatbed scanners. */
 
 
-/* $Id: snapscan-scsi.c,v 1.20 2003/01/08 21:45:15 oliverschwartz Exp $
+/* $Id: snapscan-scsi.c,v 1.22 2003/04/30 20:49:39 oliverschwartz Exp $
    SnapScan backend scsi command functions */
 
 /* scanner scsi commands */
 
 static SANE_Status download_firmware(SnapScan_Scanner * pss);
+static SANE_Status wait_scanner_ready (SnapScan_Scanner * pss);
 
 #include "snapscan-usb.h"
 
@@ -518,6 +519,7 @@ static SANE_Status inquiry (SnapScan_Scanner *pss)
 
     pss->actual_res =
         u_char_to_u_short (pss->buf + INQUIRY_OPT_RES);
+
     pss->pixels_per_line =
         u_char_to_u_short (pss->buf + INQUIRY_PIX_PER_LINE);
     pss->bytes_per_line =
@@ -742,6 +744,7 @@ static SANE_Status set_window (SnapScan_Scanner *pss)
     SANE_Status status;
     unsigned char source;
     u_char *pc;
+    int pos_factor;
 
     DBG (DL_CALL_TRACE, "%s\n", me);
     zero_buf (pss->cmd, MAX_SCSI_CMD_LEN);
@@ -763,19 +766,32 @@ static SANE_Status set_window (SnapScan_Scanner *pss)
     u_short_to_u_charp (pss->res, pc + SET_WINDOW_P_YRES);
     DBG (DL_CALL_TRACE, "%s Resolution: %d\n", me, pss->res);
 
+    pos_factor = pss->actual_res;
+    if (pss->pdev->model == PRISA5000) 
+    {
+        if (pss->res > 600) 
+        {
+            pos_factor = 1200;
+        }
+        else
+        {
+            pos_factor = 600;
+        }    
+    }
+
     /* it's an ugly sound if the scanner drives against the rear
        wall, and with changing max values we better be sure */
     check_range(&(pss->brx), pss->pdev->x_range);
     check_range(&(pss->bry), pss->pdev->y_range);
     {
         int tlxp =
-            (int) (pss->actual_res*IN_PER_MM*SANE_UNFIX(pss->tlx));
+            (int) (pos_factor*IN_PER_MM*SANE_UNFIX(pss->tlx));
         int tlyp =
-            (int) (pss->actual_res*IN_PER_MM*SANE_UNFIX(pss->tly));
+            (int) (pos_factor*IN_PER_MM*SANE_UNFIX(pss->tly));
         int brxp =
-            (int) (pss->actual_res*IN_PER_MM*SANE_UNFIX(pss->brx));
+            (int) (pos_factor*IN_PER_MM*SANE_UNFIX(pss->brx));
         int bryp =
-            (int) (pss->actual_res*IN_PER_MM*SANE_UNFIX(pss->bry));
+            (int) (pos_factor*IN_PER_MM*SANE_UNFIX(pss->bry));
 
         /* Check for brx > tlx and bry > tly */
         if (brxp <= tlxp) {
@@ -876,8 +892,15 @@ static SANE_Status set_window (SnapScan_Scanner *pss)
     pc[SET_WINDOW_P_BLUE_UNDER_COLOR] = 0xff;
     pc[SET_WINDOW_P_GREEN_UNDER_COLOR] = 0xff;
 
-    status = snapscan_cmd (pss->pdev->bus, pss->fd, pss->cmd,
-               SET_WINDOW_TOTAL_LEN, NULL, NULL);
+    do {
+        status = snapscan_cmd (pss->pdev->bus, pss->fd, pss->cmd,
+                  SET_WINDOW_TOTAL_LEN, NULL, NULL);
+        if (status == SANE_STATUS_DEVICE_BUSY) {
+            DBG (DL_MINOR_INFO, "%s: waiting for scanner to warm up\n", me);
+            wait_scanner_ready (pss);
+        }
+    } while (status == SANE_STATUS_DEVICE_BUSY);
+
     CHECK_STATUS (status, me, "snapscan_cmd");
     return status;
 }
@@ -957,7 +980,9 @@ static SANE_Status send_diagnostic (SnapScan_Scanner *pss)
         ||
 	pss->pdev->model == VUEGO610S
 	||
-	pss->pdev->model == SNAPSCAN1236) 
+	pss->pdev->model == SNAPSCAN1236
+        ||
+        pss->pdev->model == ARCUS1200) 
     {
         return SANE_STATUS_GOOD;
     }
@@ -1186,8 +1211,17 @@ static SANE_Status download_firmware(SnapScan_Scanner * pss)
 
 /*
  * $Log: snapscan-scsi.c,v $
- * Revision 1.20  2003/01/08 21:45:15  oliverschwartz
- * Update to snapscan backend 1.4.18
+ * Revision 1.22  2003/04/30 20:49:39  oliverschwartz
+ * SnapScan backend 1.4.26
+ *
+ * Revision 1.37  2003/04/30 20:42:19  oliverschwartz
+ * Added support for Agfa Arcus 1200 (supplied by Valtteri Vuorikoski)
+ *
+ * Revision 1.36  2003/04/02 21:17:13  oliverschwartz
+ * Fix for 1200 DPI with Acer 5000
+ *
+ * Revision 1.35  2003/02/08 10:45:09  oliverschwartz
+ * Use 600 DPI as optical resolution for Benq 5000
  *
  * Revision 1.34  2002/12/10 20:14:12  oliverschwartz
  * Enable color offset correction for SnapScan300
