@@ -47,7 +47,7 @@
    of the the AGFA SnapScan and Acer Vuego/Prisa flatbed scanners. */
 
 
-/* $Id: snapscan.c,v 1.27 2003/01/08 21:45:15 oliverschwartz Exp $
+/* $Id: snapscan.c,v 1.29 2003/04/30 20:49:40 oliverschwartz Exp $
    SANE SnapScan backend */
 
 #include "../include/sane/config.h"
@@ -78,7 +78,7 @@
 
 #define EXPECTED_MAJOR       1
 #define MINOR_VERSION        4
-#define BUILD               18
+#define BUILD               26
 
 #include "snapscan.h"
 
@@ -220,7 +220,15 @@ static inline int is_colour_mode (SnapScan_Mode m)
 
 static inline int calibration_line_length(SnapScan_Scanner *pss)
 {
-    int pixel_length = pss->actual_res * 8.5;
+    int pos_factor = pss->actual_res;
+    int pixel_length; 
+
+    if (pss->pdev->model == PRISA5000) 
+    {
+        pos_factor = 600;
+    }
+
+    pixel_length = pos_factor * 8.5;
 
     if(is_colour_mode(actual_mode(pss))) {
         return 3 * pixel_length;
@@ -1754,8 +1762,17 @@ SANE_Status sane_get_select_fd (SANE_Handle h, SANE_Int * fd)
 
 /*
  * $Log: snapscan.c,v $
- * Revision 1.27  2003/01/08 21:45:15  oliverschwartz
- * Update to snapscan backend 1.4.18
+ * Revision 1.29  2003/04/30 20:49:40  oliverschwartz
+ * SnapScan backend 1.4.26
+ *
+ * Revision 1.58  2003/04/30 20:43:07  oliverschwartz
+ * Set backend version number to 1.4.26
+ *
+ * Revision 1.57  2003/04/02 21:17:14  oliverschwartz
+ * Fix for 1200 DPI with Acer 5000
+ *
+ * Revision 1.56  2003/02/08 10:45:09  oliverschwartz
+ * Use 600 DPI as optical resolution for Benq 5000
  *
  * Revision 1.55  2003/01/08 21:16:17  oliverschwartz
  * Added support for Acer / Benq 310U
