@@ -65,6 +65,10 @@
 
 /* --------------------------------------------------------------------------------------------------------- */
 
+#define SANE_UMAX_FIX_ROUND(val) ((SANE_Word) ((val) * (1 << SANE_FIXED_SCALE_SHIFT) + 1.0 / (1 << (SANE_FIXED_SCALE_SHIFT+1))))
+
+/* --------------------------------------------------------------------------------------------------------- */
+
 enum Umax_Option
 {
     OPT_NUM_OPTS = 0,
@@ -495,7 +499,8 @@ typedef struct Umax_Scanner
   SANE_Parameters		params;
 
   pid_t				reader_pid;
-  int				pipe;
+  int				pipe_read_fd;
+  int				pipe_write_fd;
 } Umax_Scanner;
 
 
