@@ -1,4 +1,5 @@
 /* sane - Scanner Access Now Easy.
+   Copyright (C) 2003 Rene Rebe (sanei_read_int)
    Copyright (C) 2001, 2002 Henning Meier-Geinitz
    Copyright (C) 2001 Frank Zago (sanei_usb_control_msg)
    This file is part of the SANE package.
@@ -82,6 +83,8 @@ typedef struct
   SANE_Int product;
   SANE_Int bulk_in_ep;
   SANE_Int bulk_out_ep;
+  SANE_Int int_in_ep;
+  SANE_Int int_out_ep;
   SANE_Int interface_nr;
 #ifdef HAVE_LIBUSB
   usb_dev_handle *libusb_handle;
@@ -129,6 +132,50 @@ struct ctrlmsg_ioctl
 cmsg;
 #endif /* __linux__ */
 
+static SANE_Bool inited = SANE_FALSE;
+
+/* Debug level from sanei_init_debug */
+static SANE_Int debug_level;
+
+static void
+print_buffer (const SANE_Byte * buffer, SANE_Int size)
+{
+#define NUM_COLUMNS 16
+#define PRINT_BUFFER_SIZE (6 + NUM_COLUMNS * (3 + 1) + 1 + 1)
+  char line_str [PRINT_BUFFER_SIZE];
+  char *pp;
+  int column;
+  int line;
+
+  memset (line_str, 0, PRINT_BUFFER_SIZE);
+
+  for (line = 0; line < ((size + NUM_COLUMNS - 1) / NUM_COLUMNS); line++)
+    {
+      pp = line_str;
+      sprintf (pp, "%04X: ", line * NUM_COLUMNS);
+      pp += 6;
+      for (column = 0; column < NUM_COLUMNS; column++)
+	{
+	  if ((line * NUM_COLUMNS + column) < size)
+	    sprintf (pp, "%02X ", buffer [line * NUM_COLUMNS + column]);
+	  else
+	    sprintf (pp, "   ");
+	  pp += 3;
+	}
+      for (column = 0; column < NUM_COLUMNS; column++)
+	{
+	  if ((line * NUM_COLUMNS + column) < size)
+	    sprintf (pp, "%c",
+		     (buffer [line * NUM_COLUMNS + column] < 127) && 
+		     (buffer [line * NUM_COLUMNS + column] > 31) ?
+		     buffer [line * NUM_COLUMNS + column] : '.');
+	  else
+	    sprintf (pp, ".");
+	  pp += 1;
+	}
+      DBG(11, "%s\n", line_str);
+    }
+}
 
 static void
 kernel_get_vendor_product (int fd, int *vendorID, int *productID)
@@ -173,7 +220,18 @@ sanei_usb_init (void)
   struct usb_device *dev;
 #endif /* HAVE_LIBUSB */
 
+  if (inited)
+    return;
+
+  inited = SANE_TRUE;
+
   DBG_INIT ();
+#ifdef DBG_LEVEL
+  debug_level = DBG_LEVEL;
+#else
+  debug_level = 0;
+#endif
+
   memset (devices, 0, sizeof (devices));
 
   /* Check for devices using the kernel scanner driver */
@@ -213,7 +271,7 @@ sanei_usb_init (void)
 	      fd = open (devname, O_RDWR);
 	      if (fd < 0)
 		{
-		  DBG(5, "sanei_usb_init: couldn't open %s: %s\n", devname,
+		  DBG (5, "sanei_usb_init: couldn't open %s: %s\n", devname,
 		      strerror (errno));
 		  continue;
 		}
@@ -255,6 +313,9 @@ sanei_usb_init (void)
     {
       for (dev = bus->devices; dev; dev = dev->next)
 	{
+	  int interface;
+	  SANE_Bool found;
+
 	  if (!dev->config)
 	    {
 	      DBG (1, "sanei_usb_init: device 0x%04x/0x%04x is not configured\n",
@@ -267,29 +328,36 @@ sanei_usb_init (void)
 		   dev->descriptor.idVendor, dev->descriptor.idProduct);
 	      continue;
 	    }
-	  switch (dev->descriptor.bDeviceClass)
+	  found = SANE_FALSE;
+	  for (interface = 0; interface < dev->config[0].bNumInterfaces && !found; interface++)
 	    {
-	    case USB_CLASS_VENDOR_SPEC:
-	      break;
-	    case USB_CLASS_PER_INTERFACE:
-	      switch (dev->config[0].interface[0].altsetting[0].bInterfaceClass)
+	      switch (dev->descriptor.bDeviceClass)
 		{
 		case USB_CLASS_VENDOR_SPEC:
+		  found = SANE_TRUE;
+		  break;
 		case USB_CLASS_PER_INTERFACE:
-		case 16:                /* data? */
+		  switch (dev->config[0].interface[interface].altsetting[0].bInterfaceClass)
+		    {
+		    case USB_CLASS_VENDOR_SPEC:
+		    case USB_CLASS_PER_INTERFACE:
+		    case 16:                /* data? */
+		      found = SANE_TRUE;
+		      break;
+		    }
 		  break;
-		default:
-		  DBG (5, "sanei_usb_init: device 0x%04x/0x%04x doesn't look like a "
-		       "scanner (%d/%d)\n", dev->descriptor.idVendor,
-		       dev->descriptor.idProduct, dev->descriptor.bDeviceClass, 
-		       dev->config[0].interface[0].altsetting[0].bInterfaceClass);
-		  continue;
 		}
-	      break;
-	    default:
-	      DBG (5, "sanei_usb_init: device 0x%04x/0x%04x doesn't look like a scanner (%d)\n",
-		   dev->descriptor.idVendor, dev->descriptor.idProduct,
-		   dev->descriptor.bDeviceClass);
+	      if (!found)
+		DBG (5, "sanei_usb_init: device 0x%04x/0x%04x, interface %d doesn't look like a "
+		     "scanner (%d/%d)\n", dev->descriptor.idVendor,
+		     dev->descriptor.idProduct, interface, dev->descriptor.bDeviceClass, 
+		     dev->config[0].interface[interface].altsetting[0].bInterfaceClass);
+	    }
+	  interface--;
+	  if (!found)
+	    {
+	      DBG (5, "sanei_usb_init: device 0x%04x/0x%04x: no suitable interfaces\n",
+		   dev->descriptor.idVendor, dev->descriptor.idProduct);
 	      continue;
 	    }
 	  
@@ -303,8 +371,9 @@ sanei_usb_init (void)
 	  devices[dn].product = dev->descriptor.idProduct;
 	  devices[dn].method = sanei_usb_method_libusb;
 	  devices[dn].open = SANE_FALSE;
-	  DBG (4, "sanei_usb_init: found libusb device (0x%04x/0x%04x) at %s\n",
-	       dev->descriptor.idVendor, dev->descriptor.idProduct, devname);
+	  devices[dn].interface_nr = interface;
+	  DBG (4, "sanei_usb_init: found libusb device (0x%04x/0x%04x) interface %d  at %s\n",
+	       dev->descriptor.idVendor, dev->descriptor.idProduct, interface, devname);
 	  dn++;
 	  if (dn >= MAX_DEVICES)
 	    return;
@@ -530,15 +599,6 @@ sanei_usb_open (SANE_String_Const devname, SANE_Int * dn)
 	}
       
       /* Claim the interface */
-      devices[devcount].interface_nr = 
-	dev->config[0].interface[0].altsetting[0].bInterfaceNumber;
-      if (dev->config[0].bNumInterfaces > 1)
-	{
-	  DBG (3, "sanei_usb_open: more than one "
-	       "interface (%d), choosing first interface (%d)\n",
-	       dev->config[0].bNumInterfaces, 
-	       devices[devcount].interface_nr);
-	}
       result = usb_claim_interface (devices[devcount].libusb_handle, 
 				    devices[devcount].interface_nr);
       if (result < 0)
@@ -604,8 +664,53 @@ sanei_usb_open (SANE_String_Const devname, SANE_Int * dn)
 	  direction = endpoint->bEndpointAddress & USB_ENDPOINT_DIR_MASK;
 	  transfer_type = endpoint->bmAttributes & USB_ENDPOINT_TYPE_MASK;
 
-	  if (transfer_type != USB_ENDPOINT_TYPE_BULK)
+	  /* save the endpoints we need later */
+	  if (transfer_type == USB_ENDPOINT_TYPE_INTERRUPT)
+	  {
+	    DBG (5, "sanei_usb_open: found interupt-%s endpoint (address %d)\n",
+	         direction ? "in" : "out", address);
+	    if (direction)	/* in */
 	    {
+	      if (devices[devcount].int_in_ep)
+		DBG (3, "sanei_usb_open: we already have a int-in endpoint "
+		     "(address: %d), ignoring the new one\n",
+		     devices[devcount].int_in_ep);
+	      else
+		devices[devcount].int_in_ep = endpoint->bEndpointAddress;
+	    }
+	    else
+	      if (devices[devcount].int_out_ep)
+		DBG (3, "sanei_usb_open: we already have a int-out endpoint "
+		     "(address: %d), ignoring the new one\n",
+		     devices[devcount].int_out_ep);
+	      else
+		devices[devcount].int_out_ep = endpoint->bEndpointAddress;
+	  }
+	  else if (transfer_type == USB_ENDPOINT_TYPE_BULK)
+	  {
+	    DBG (5, "sanei_usb_open: found bulk-%s endpoint (address %d)\n",
+	         direction ? "in" : "out", address);
+	    if (direction)	/* in */
+	      {
+		if (devices[devcount].bulk_in_ep)
+		  DBG (3, "sanei_usb_open: we already have a bulk-in endpoint "
+		       "(address: %d), ignoring the new one\n",
+		       devices[devcount].bulk_in_ep);
+		else
+		  devices[devcount].bulk_in_ep = endpoint->bEndpointAddress;
+	      }
+	    else
+	      {
+	        if (devices[devcount].bulk_out_ep)
+		  DBG (3, "sanei_usb_open: we already have a bulk-out endpoint "
+		       "(address: %d), ignoring the new one\n",
+		       devices[devcount].bulk_out_ep);
+	        else
+		  devices[devcount].bulk_out_ep = endpoint->bEndpointAddress;
+	      }
+	    }
+	  /* ignore currently unsupported endpoints */
+	  else {
 	      DBG (5, "sanei_usb_open: ignoring %s-%s endpoint "
 		   "(address: %d)\n",
 		   transfer_type == USB_ENDPOINT_TYPE_CONTROL ? "control" :
@@ -614,26 +719,6 @@ sanei_usb_open (SANE_String_Const devname, SANE_Int * dn)
 		   direction ? "in" : "out", address);
 	      continue;
 	    }
-	  DBG (5, "sanei_usb_open: found bulk-%s endpoint (address %d)\n",
-	       direction ? "in" : "out", address);
-	  if (direction)	/* in */
-	    {
-	      if (devices[devcount].bulk_in_ep)
-		DBG (3, "sanei_usb_open: we already have a bulk-in endpoint "
-		     "(address: %d), ignoring the new one\n",
-		     devices[devcount].bulk_in_ep);
-	      else
-		devices[devcount].bulk_in_ep = endpoint->bEndpointAddress;
-	    }
-	  else
-	    {
-	      if (devices[devcount].bulk_out_ep)
-		DBG (3, "sanei_usb_open: we already have a bulk-out endpoint "
-		     "(address: %d), ignoring the new one\n",
-		     devices[devcount].bulk_out_ep);
-	      else
-		devices[devcount].bulk_out_ep = endpoint->bEndpointAddress;
-	    }
 	}
 #else /* not HAVE_LIBUSB */
       DBG (1, "sanei_usb_open: can't open device `%s': "
@@ -730,6 +815,9 @@ sanei_usb_read_bulk (SANE_Int dn, SANE_Byte * buffer, size_t * size)
       DBG (1, "sanei_usb_read_bulk: dn >= MAX_DEVICES || dn < 0\n");
       return SANE_STATUS_INVAL;
     }
+  DBG (5, "sanei_usb_read_bulk: trying to read %lu bytes\n",
+       (unsigned long) *size);
+
   if (devices[dn].method == sanei_usb_method_scanner_driver)
     read_size = read (devices[dn].fd, buffer, *size);
   else if (devices[dn].method == sanei_usb_method_libusb)
@@ -763,7 +851,8 @@ sanei_usb_read_bulk (SANE_Int dn, SANE_Byte * buffer, size_t * size)
     {
       DBG (1, "sanei_usb_read_bulk: read failed: %s\n", strerror (errno));
 #ifdef HAVE_LIBUSB
-      usb_clear_halt (devices[dn].libusb_handle, devices[dn].bulk_in_ep);
+      if (devices[dn].method == sanei_usb_method_libusb)
+	usb_clear_halt (devices[dn].libusb_handle, devices[dn].bulk_in_ep);
 #endif
       *size = 0;
       return SANE_STATUS_IO_ERROR;
@@ -774,9 +863,12 @@ sanei_usb_read_bulk (SANE_Int dn, SANE_Byte * buffer, size_t * size)
       *size = 0;
       return SANE_STATUS_EOF;
     }
+  if (debug_level > 10)
+    print_buffer (buffer, read_size);
   DBG (5, "sanei_usb_read_bulk: wanted %lu bytes, got %ld bytes\n",
        (unsigned long) *size, (unsigned long) read_size);
   *size = read_size;
+    
   return SANE_STATUS_GOOD;
 }
 
@@ -796,6 +888,10 @@ sanei_usb_write_bulk (SANE_Int dn, const SANE_Byte * buffer, size_t * size)
       DBG (1, "sanei_usb_write_bulk: dn >= MAX_DEVICES || dn < 0\n");
       return SANE_STATUS_INVAL;
     }
+  DBG (5, "sanei_usb_write_bulk: trying to write %lu bytes\n",
+       (unsigned long) *size);
+  if (debug_level > 10)
+    print_buffer (buffer, *size);
 
   if (devices[dn].method == sanei_usb_method_scanner_driver)
     write_size = write (devices[dn].fd, buffer, *size);
@@ -832,7 +928,8 @@ sanei_usb_write_bulk (SANE_Int dn, const SANE_Byte * buffer, size_t * size)
       DBG (1, "sanei_usb_write_bulk: write failed: %s\n", strerror (errno));
       *size = 0;
 #ifdef HAVE_LIBUSB
-      usb_clear_halt (devices[dn].libusb_handle, devices[dn].bulk_out_ep);
+      if (devices[dn].method == sanei_usb_method_libusb)
+	usb_clear_halt (devices[dn].libusb_handle, devices[dn].bulk_out_ep);
 #endif
       return SANE_STATUS_IO_ERROR;
     }
@@ -853,6 +950,11 @@ sanei_usb_control_msg (SANE_Int dn, SANE_Int rtype, SANE_Int req,
       return SANE_STATUS_INVAL;
     }
 
+  DBG (5, "sanei_usb_control_msg: rtype = 0x%02x, req = %d, value = %d, "
+       "index = %d, len = %d\n", rtype, req, value, index, len);
+  if (!(rtype & 0x80) && debug_level > 10)
+    print_buffer (data, len);
+
   if (devices[dn].method == sanei_usb_method_scanner_driver)
     {
 #if defined(__linux__)
@@ -865,15 +967,14 @@ sanei_usb_control_msg (SANE_Int dn, SANE_Int rtype, SANE_Int req,
       c.req.length = len;
       c.data = data;
 
-      DBG (5, "sanei_usb_control_msg: rtype = 0x%02x, req = %d, value = %d, "
-	   "index = %d, len = %d\n", rtype, req, value, index, len);
-
       if (ioctl (devices[dn].fd, SCANNER_IOCTL_CTRLMSG, &c) < 0)
 	{
 	  DBG (5, "sanei_usb_control_msg: SCANNER_IOCTL_CTRLMSG error - %s\n",
 	       strerror (errno));
 	  return SANE_STATUS_IO_ERROR;
 	}
+      if ((rtype & 0x80) && debug_level > 10)
+	print_buffer (data, len);
       return SANE_STATUS_GOOD;
 #else /* not __linux__ */
       DBG (5, "sanei_usb_control_msg: not supported on this OS\n");
@@ -894,6 +995,8 @@ sanei_usb_control_msg (SANE_Int dn, SANE_Int rtype, SANE_Int req,
 	       usb_strerror ());
 	  return SANE_STATUS_INVAL;
 	}
+      if ((rtype & 0x80) && debug_level > 10)
+	print_buffer (data, len);
       return SANE_STATUS_GOOD;
     }
 #else /* not HAVE_LIBUSB */
@@ -904,8 +1007,80 @@ sanei_usb_control_msg (SANE_Int dn, SANE_Int rtype, SANE_Int req,
 #endif /* not HAVE_LIBUSB */
   else
     {
-      DBG (1, "sanei_usb_read_bulk: access method %d not implemented\n",
+      DBG (1, "sanei_usb_control_msg: access method %d not implemented\n",
 	   devices[dn].method);
       return SANE_STATUS_UNSUPPORTED;
     }
 }
+
+SANE_Status
+sanei_usb_read_int (SANE_Int dn, SANE_Byte * buffer, size_t * size)
+{
+  ssize_t read_size = 0;
+
+  if (!size)
+    {
+      DBG (1, "sanei_usb_read_int: size == NULL\n");
+      return SANE_STATUS_INVAL;
+    }
+
+  if (dn >= MAX_DEVICES || dn < 0)
+    {
+      DBG (1, "sanei_usb_read_int: dn >= MAX_DEVICES || dn < 0\n");
+      return SANE_STATUS_INVAL;
+    }
+  if (devices[dn].method == sanei_usb_method_scanner_driver) {
+     DBG (1, "sanei_usb_read_int: access method %d not implemented\n",
+	   devices[dn].method);
+     return SANE_STATUS_INVAL;
+  }
+  else if (devices[dn].method == sanei_usb_method_libusb)
+#ifdef HAVE_LIBUSB
+    {
+      if (devices[dn].int_in_ep)
+	read_size = usb_bulk_read (devices[dn].libusb_handle,
+				   devices[dn].int_in_ep, (char *) buffer,
+				   (int) *size, libusb_timeout);
+      else
+	{
+	  DBG (1, "sanei_usb_read_int: can't read without an int "
+	       "endpoint\n");
+	  return SANE_STATUS_INVAL;
+	}
+    }
+#else /* not HAVE_LIBUSB */
+    {
+      DBG (1, "sanei_usb_read_int: libusb support missing\n");
+      return SANE_STATUS_UNSUPPORTED;
+    }
+#endif /* not HAVE_LIBUSB */
+  else
+    {
+      DBG (1, "sanei_usb_read_int: access method %d not implemented\n",
+	   devices[dn].method);
+      return SANE_STATUS_INVAL;
+    }
+
+  if (read_size < 0)
+    {
+      DBG (1, "sanei_usb_read_int: read failed: %s\n", strerror (errno));
+#ifdef HAVE_LIBUSB
+      if (devices[dn].method == sanei_usb_method_libusb)
+	usb_clear_halt (devices[dn].libusb_handle, devices[dn].int_in_ep);
+#endif
+      *size = 0;
+      return SANE_STATUS_IO_ERROR;
+    }
+  if (read_size == 0)
+    {
+      DBG (3, "sanei_usb_read_int: read returned EOF\n");
+      *size = 0;
+      return SANE_STATUS_EOF;
+    }
+  DBG (5, "sanei_usb_read_int: wanted %lu bytes, got %ld bytes\n",
+       (unsigned long) *size, (unsigned long) read_size);
+  *size = read_size;
+  if (debug_level > 10)
+    print_buffer (buffer, read_size);
+  return SANE_STATUS_GOOD;
+}
